/**
 * 
 */
package com.as.lyons.res.svc.aoa.client;

import org.slf4j.helpers.MessageFormatter;
import org.springframework.util.Assert;
import org.springframework.web.client.HttpServerErrorException;
import org.springframework.web.client.RestTemplate;

import com.as.lyons.rest.svc.aoa.AccountStatusRequest;
import com.as.lyons.rest.svc.common.ErrorResult;
import com.as.lyons.rest.svc.common.LogonRequest;
import com.as.lyons.rest.svc.common.LogonResult;
import com.as.lyons.rest.svc.common.TokenRequest;
import com.fasterxml.jackson.databind.ObjectMapper;

/**
 * @author pgurov
 *
 */
public abstract class AbstractAoaServiceCall implements AoaServiceCall {

	protected void handleError(String msg, String errorMessage, boolean rethrow) throws StopProcesingException {
		String resMsg = MessageFormatter.arrayFormat("{}{}{}", new String[] { msg, errorMessage == null ? " - " : ": ",
				errorMessage == null ? "got a REST exception" : errorMessage }).getMessage();
		System.out.println(resMsg);
		if (rethrow) {
			throw new StopProcesingException();
		}
	}

	protected void handleRestException(String msg, HttpServerErrorException ex, boolean rethrow)
			throws StopProcesingException {
		String res = null;
		try {
			String obj = ex.getResponseBodyAsString();
			ObjectMapper om = new ObjectMapper();
			ErrorResult err = om.readValue(obj, ErrorResult.class);
			res = err.getErrorMessage();
		} catch (Exception e) {
		}
		handleError(msg, res, rethrow);
	}

	protected void message(String template, Object... args) {
		String msg = MessageFormatter.arrayFormat(template, args).getMessage();
		System.out.println(msg);
	}

	protected String login(RestTemplate rt, LogonProperties loginProps) throws StopProcesingException {
		LogonRequest req = new LogonRequest();
		req.setCompanyId(loginProps.getCompanyId());
		req.setUserName(loginProps.getUserName());
		req.setPassword(loginProps.getPassword());
		message("Logging in to AOA service with companyId={} and userName={}", loginProps.getCompanyId(),
				loginProps.getUserName());
		String token = null;
		try {
			LogonResult res = rt.postForObject("/logon", req, LogonResult.class);
			message("Executed the Logon request, the result is {}null", res == null ? "" : "not ");
			Assert.notNull(res, "The response is null");
			token = res.getToken();
			Assert.notNull(token, "The returned token is null");
			message("The token is: {}", token);
		} catch (HttpServerErrorException ex) {
			handleRestException("\nError logging on to IAOAService 1", ex, true);
		} catch (Exception e) {
			handleError("\nError logging on to IAOAService 2", e.getMessage(), true);
		}
		return token;
	}

	protected void logoff(RestTemplate rt, String token) {
		TokenRequest req = new TokenRequest();
		req.setToken(token);
		message("Logging off session: {}", token);
		try {
			try {
				rt.postForObject("/logoff", req, Object.class);
			} catch (HttpServerErrorException ex) {
				handleRestException("\nError logging off IAOAService 1", ex, false);
			} catch (Exception e) {
				handleError("\nError logging off IAOAService 2", e.getMessage(), false);
			}
		} catch (StopProcesingException e) {

		}
	}
	
	protected AccountStatusRequest prepareAccountStatusRequest(String[] args, LogonProperties loginProps) {
		
		AccountStatusRequest accountStatusRequest = new AccountStatusRequest();
		accountStatusRequest.setRtn(args[1]);
		accountStatusRequest.setAccountNo(args[2]);
		accountStatusRequest.setAmount(args[3]);
		accountStatusRequest.setCustomerPresent(args[4]);
		accountStatusRequest.setDenyNsf(args[5]);
		accountStatusRequest.setCountry(args[6]);
		accountStatusRequest.setAccountOwner(loginProps.getOwner());
		return accountStatusRequest;
	}
	
}
