/**
 * 
 */
package com.as.lyons.res.svc.aoa.client;
import org.springframework.web.client.HttpServerErrorException;
import org.springframework.web.client.RestTemplate;
import com.as.lyons.rest.svc.aoa.AccountRequest;
import com.as.lyons.rest.svc.aoa.AccountStatusRequest;
import com.as.lyons.rest.svc.aoa.DetailedStatusCodeResult;
import com.as.lyons.rest.svc.aoa.InstitutionAddress;

/**
 * @author pgurov
 *
 */
public class CheckAccountOwnershipAndStatusCall extends AbstractAoaServiceCall {

	@Override
	public void printHelp(LogonProperties loginProps, String method) {
		StringBuilder sb = new StringBuilder(500).append("Use:\n\t").append(loginProps.getAppStartCommand());
		sb.append(method).append("<returnDetails: 0|1> <routing number> <account number> <amount e.g. 123.12> <customer present flag: 0|1> <deny non sufficent funds(nsf) flag: 0|1>  <country name e.g.'US'>");
		sb.append(method).append("Enter account owner mandatory fields details in application properties folder");
		System.out.println(sb.toString());
	}

	@Override
	public void call(RestTemplate rt, LogonProperties loginProps, String method, String[] args)
			throws StopProcesingException {
		AccountRequest req = prepareRequest(loginProps, method, args);
		String token = login(rt, loginProps);
		DetailedStatusCodeResult res = null;
		try {
			req.setToken(token);
			res = rt.postForObject("/".concat(method), req, DetailedStatusCodeResult.class);
		} catch (HttpServerErrorException ex) {
			handleRestException("\nError calling " + method + " on the AOA service 1", ex, false);
		} catch (Exception e) {
			handleError("\nError calling " + method + " on the AOA service 2", e.getMessage(), false);
		}
		reportResult(method, res);
		logoff(rt, token);
	}

	private void reportResult(String method, DetailedStatusCodeResult res) {
		if (res == null) {
			message("The AOA {} call did not return a response - this is an error!", method);
		} else {
			InstitutionAddress fi = res.getPrimaryInstitution();
			if (fi == null) {
				// this is generally an error, but for the test
				fi = new InstitutionAddress();
			}
			message("The AOA {} call returned status code: {} with description {}; and FI: {} of {} {}", method, res.getStatusCode(),res.getStatusDescription(),
					fi.getName(), fi.getCity(), fi.getState());
		}
	}

	private AccountRequest prepareRequest(LogonProperties loginProps, String method, String[] args)
			throws StopProcesingException {
		if (args != null && args.length - 1 != 6) {
			printHelp(loginProps, method);
			throw new StopProcesingException();
		}
	
		AccountStatusRequest accountStatusRequest = prepareAccountStatusRequest(args,loginProps);
		AccountRequest req = new AccountRequest(null, args[0], accountStatusRequest);
		return req;
	}
}
