/**
 * 
 */
package com.as.lyons.res.svc.aoa.client;

import org.springframework.web.client.HttpServerErrorException;
import org.springframework.web.client.RestTemplate;

import com.as.lyons.rest.svc.aoa.DescribeStatusCodeRequest;
import com.as.lyons.rest.svc.aoa.StatusDescriptionResult;

/**
 * @author pgurov
 *
 */
public class DescribeStatusCodeServiceCall extends AbstractAoaServiceCall {

	@Override
	public void printHelp(LogonProperties loginProps, String method) {
		StringBuilder sb = new StringBuilder(500).append("Use:\n\t").append(loginProps.getAppStartCommand());
		sb.append(method).append(" <an integer status code>");
		System.out.println(sb.toString());
	}

	@Override
	public void call(RestTemplate rt, LogonProperties loginProps, String method, String[] args)
			throws StopProcesingException {
		DescribeStatusCodeRequest req = prepareRequest(loginProps, method, args);
		String token = login(rt, loginProps);
		StatusDescriptionResult res = null;
		try {
			req.setToken(token);
			res = rt.postForObject("/".concat(method), req, StatusDescriptionResult.class);
		} catch (HttpServerErrorException ex) {
			handleRestException("\nError calling " + method + " on the AOA service 1", ex, false);
		} catch (Exception e) {
			handleError("\nError calling " + method + " on the AOA service 2", e.getMessage(), false);
		}
		reportResult(method, res);
		logoff(rt, token);
	}

	private void reportResult(String method, StatusDescriptionResult res) {
		if(res == null) {
			message("The AOA {} call did not return a response - this is an error!", method);
		} else {
			message("The Aoa {} returned the following status description: {}", method, res.getStatusDescription());
		}
	}

	private DescribeStatusCodeRequest prepareRequest(LogonProperties loginProps, String method, String[] args)
			throws StopProcesingException {
		if (args == null || args.length == 0) {
			printHelp(loginProps, method);
		
			throw new StopProcesingException();
		}
		int statusCode = 0;
		try {
			statusCode = Integer.parseInt(args[0]);
		} catch(Exception e) {
			printHelp(loginProps, method);
			throw new StopProcesingException();
		}
		DescribeStatusCodeRequest req = new DescribeStatusCodeRequest(null, statusCode);
		return req;
		
	}

}
